#include "stdafx.h"
#include "pack.h"

#define SAMPLERATE		22050
#define BYTERATE		44100
#define BPS				16
#define BLOCKALIGN		2

SND_HEADER header = {0};

static void pack_append(SND_DESC* desc)
{
	SND_DESC* next = header.first;
	SND_DESC* prev;

	while (next)
	{
		prev = next;
		next = next->next;
	}

	if (next == header.first)
	{
		header.first = desc;
		header.cnt = 1;
	} else {
		prev->next = desc;
		header.cnt++;
	}
}

void pack_new()
{
	SND_DESC* desc = header.first;
	SND_DESC* next;

	while (desc)
	{
		next = desc->next;

		if (desc->allocd)
			free(desc->data);

		free(desc);
		desc = next;
	}

	memset(&header, 0, sizeof(header));
}

void pack_open(BYTE* data, DWORD len)
{
	BYTE cnt;
	DWORD data_offs = 0;
	SND_DESC* prev = NULL;
	SND_DESC* desc = NULL;

	if (len < 5) return;

	pack_new();

	memcpy(&header.ver, data, 2);

	if (header.ver != 1) return;

	header.cnt = *(data+2);

	if (!header.cnt) return;

	memcpy(&header.crc, data+3, 2);

	header.first = NULL;

	cnt = header.cnt;

	data_offs = 5;

	while (cnt)
	{
		desc = (SND_DESC*) malloc(sizeof(SND_DESC));
		if (!desc) return;

		desc->type = *(data+(data_offs++));

		memcpy(&desc->offs, data+data_offs, sizeof(DWORD));
		data_offs += 4;

		memcpy(&desc->len, data+data_offs, sizeof(DWORD));
		data_offs += 4;

		desc->data = (BYTE*) malloc(desc->len);
		if (!desc->data)
		{
			pack_new();
			return;
		}
		memcpy(desc->data, data+desc->offs, desc->len);

		desc->next = NULL;

		if (header.first == NULL)
		{
			header.first = desc;
			prev = header.first;
		} else {
			prev->next = desc;
			prev = desc;
		}

		cnt--;
	}
}

BOOL pack_add(BYTE* buffer, DWORD len, BYTE type)
{
	DWORD offs = 0;
	DWORD val32;
	USHORT val16;
	SND_DESC* desc;
	DWORD fmt_chunk_size;

	/* RIFF */
	if (memcmp(buffer, "RIFF", 4) != 0)
		return FALSE;

	buffer += 4;

	/* Chunk size */
	memcpy(&val32, buffer, 4);
	if (val32 != len - 8) return FALSE;

	/* Skip fmt */
	buffer += 4 + 8;

	/* Chunk size */
	memcpy(&fmt_chunk_size, buffer, 4);
	if (fmt_chunk_size != 16 && fmt_chunk_size != 18) return FALSE;

	buffer += 4;

	/* PCM */
	memcpy(&val16, buffer, 2);
	if (val16 != 1) return FALSE;

	buffer += 2;

	/* Channels */
	memcpy(&val16, buffer, 2);
	if (val16 != 1) return FALSE;

	buffer += 2;

	/* Sample rate */
	memcpy(&val32, buffer, 4);
	if (val32 != SAMPLERATE) return FALSE;

	buffer += 4;

	/* Byte rate */
	memcpy(&val32, buffer, 4);
	if (val32 != BYTERATE) return FALSE;

	buffer += 4;

	/* Block align */
	memcpy(&val16, buffer, 2);
	if (val16 != BLOCKALIGN) return FALSE;
	
	buffer += 2;

	/* Bits per sample */
	memcpy(&val16, buffer, 2);
	if (val16 != BPS) return FALSE;
	
	buffer += 2;

	if (fmt_chunk_size == 18)
		buffer += 2;

	/* RIFF */
	if (memcmp(buffer, "data", 4) != 0)
	{
		buffer += 4;
		memcpy(&val32, buffer, 4);

		buffer += 4 + val32;

		if (memcmp(buffer, "data", 4) != 0)
			return FALSE;
	}

	buffer += 4;

	/* Data len */
	memcpy(&val32, buffer, 4);

	buffer += 4;

	desc = (SND_DESC*) malloc(sizeof(SND_DESC));
	if (!desc) return FALSE;

	desc->allocd = TRUE;
	desc->len = val32;
	desc->next = NULL;
	desc->type = type;
	desc->data = (BYTE*) malloc(desc->len);
	desc->offs = 0;
	if (!desc->data)
	{
		free(desc);
		return FALSE;
	}

	memcpy(desc->data, buffer, desc->len);

	pack_append(desc);
	return TRUE;
}

DWORD pack_calculate_size()
{
	DWORD size = 0;
	SND_DESC* desc = header.first;

	size = 5; // hdr
	while (desc)
	{
		size += 9;
		size += desc->len;
		desc = desc->next;
	}

	return size;
}

DWORD pack_get_blob(BYTE* buffer)
{
	DWORD offs = 0;
	DWORD data_offs = 0;
	SND_DESC* desc = header.first;
	DWORD size = pack_calculate_size();

	if (!buffer) return size;

	header.ver = 1;

	memcpy(buffer, &header, 5);

	offs += 5;
	data_offs = (header.cnt * 9) + 5;

	while (desc)
	{
		desc->offs = data_offs;
		memcpy(buffer+offs, desc, 9);
		memcpy(buffer+data_offs, desc->data, desc->len);

		offs += 9;
		data_offs += desc->len;

		desc = desc->next;
	}

	return size;
}

SND_HEADER* pack_get_header()
{
	return &header;
}

void pack_remove(int num)
{
	SND_DESC* desc;
	SND_DESC* last = NULL;

	if (num > header.cnt) return;

	desc = header.first;

	while (desc)
	{
		num--;
		if (num == 0)
		{
			if (last == NULL)
			{
				header.first = desc->next;
			} else {
				last->next = desc->next;
			}

			if (desc->allocd)
				free(desc->data);

			free(desc);
			header.cnt--;

			return;
		}

		last = desc;
		desc = desc->next;
		
	}
}