/***************************************************************************
 * Artekit STM32 DIY lightsaber audio board
 * Copyright (C) 2013 Artekit Italy
 * Written by Ivan Meleca
 * http://www.artekit.eu

### spi.c

#   This program is free software; you can redistribute it and/or modify
#   it under the terms of the GNU General Public License as published by
#   the Free Software Foundation; either version 2 of the License, or
#   (at your option) any later version.
#
#   This program is distributed in the hope that it will be useful,
#   but WITHOUT ANY WARRANTY; without even the implied warranty of
#   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
#   GNU General Public License for more details.
#
#   You should have received a copy of the GNU General Public License
#   along with this program; if not, write to the Free Software
#   Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.

***************************************************************************/

#include "spi.h"
#include "sys.h"

void spiInit(void)
{
	GPIO_InitTypeDef GPIO_InitStructure;
	SPI_InitTypeDef SPI_InitStructure;

	GPIO_InitStructure.GPIO_Pin = GPIO_Pin_5 | GPIO_Pin_7;
	GPIO_InitStructure.GPIO_Speed = GPIO_Speed_50MHz;
	GPIO_InitStructure.GPIO_Mode = GPIO_Mode_AF_PP;
	GPIO_Init(GPIOA, &GPIO_InitStructure);
	
	GPIO_InitStructure.GPIO_Pin = GPIO_Pin_4;
	GPIO_InitStructure.GPIO_Speed = GPIO_Speed_50MHz;
	GPIO_InitStructure.GPIO_Mode = GPIO_Mode_Out_PP;
	GPIO_Init(GPIOA, &GPIO_InitStructure);
	
	GPIO_InitStructure.GPIO_Pin = GPIO_Pin_6;
	GPIO_InitStructure.GPIO_Speed = GPIO_Speed_50MHz;
	GPIO_InitStructure.GPIO_Mode = GPIO_Mode_IN_FLOATING;
	GPIO_Init(GPIOA, &GPIO_InitStructure);
	
	SPI_InitStructure.SPI_Direction = SPI_Direction_2Lines_FullDuplex;
	SPI_InitStructure.SPI_Mode = SPI_Mode_Master;
	SPI_InitStructure.SPI_DataSize = SPI_DataSize_8b;
	SPI_InitStructure.SPI_CPOL = SPI_CPOL_Low;
	SPI_InitStructure.SPI_CPHA = SPI_CPHA_1Edge;
	SPI_InitStructure.SPI_NSS = SPI_NSS_Soft;
	SPI_InitStructure.SPI_BaudRatePrescaler = SPI_BaudRatePrescaler_2;
	SPI_InitStructure.SPI_FirstBit = SPI_FirstBit_MSB;
	SPI_InitStructure.SPI_CRCPolynomial = 7;
	SPI_Init(SPI1, &SPI_InitStructure);
	
	SPI_Cmd(SPI1, ENABLE);
	
	NVIC_EnableIRQ(DMA1_Channel2_IRQn);
	NVIC_SetPriority(DMA1_Channel2_IRQn, 14);
	
	FLASH_CS_OFF();
	DAC_CS_OFF();
}

void spiWrite(uint8_t* buffer, uint16_t len)
{
	uint16_t sent = 0;
	while (sent < len)
	{
		while (!(SPI1->SR & SPI_SR_TXE)); // Wait for bus free
		/* Send SPI_MASTER data */
		SPI_I2S_SendData(SPI1, buffer[sent++]);
	}
}

uint8_t spiWriteByte(uint8_t c)
{
	while (!(SPI1->SR & SPI_SR_TXE));
	SPI_I2S_SendData(SPI1, c);
	while (SPI_I2S_GetFlagStatus(SPI1, SPI_I2S_FLAG_RXNE) == RESET);
	return SPI_I2S_ReceiveData(SPI1);
}

void spiRead(uint8_t* buffer, uint32_t len)
{
	uint32_t read = 0;
	uint8_t dummy = 0;
	
	while (!(SPI1->SR & SPI_SR_TXE)); // Wait for bus free
	
	SPI_I2S_ReceiveData(SPI1);
	
	while (read < len)
	{
		SPI_I2S_SendData(SPI1, dummy);
		
		/*!< Wait to receive a byte */
		while (SPI_I2S_GetFlagStatus(SPI1, SPI_I2S_FLAG_RXNE) == RESET);
		
		buffer[read++] = SPI_I2S_ReceiveData(SPI1);
	}
}

uint8_t spiReadByte(void)
{
	uint8_t ret;
	
	spiRead(&ret, 1);
	
	return ret;
}
