/***************************************************************************
 * Artekit STM32 DIY lightsaber audio board
 * Copyright (C) 2013 Artekit Italy
 * Written by Ivan Meleca
 * http://www.artekit.eu
 
### sys.c
 
#   This program is free software; you can redistribute it and/or modify
#   it under the terms of the GNU General Public License as published by
#   the Free Software Foundation; either version 2 of the License, or
#   (at your option) any later version.
#
#   This program is distributed in the hope that it will be useful,
#   but WITHOUT ANY WARRANTY; without even the implied warranty of
#   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
#   GNU General Public License for more details.
#
#   You should have received a copy of the GNU General Public License
#   along with this program; if not, write to the Free Software
#   Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.

***************************************************************************/

#include "stm32f10x.h"
#include "sys.h"
#include "config.h"

volatile uint32_t sysTiming;
volatile uint32_t sysTicks = 0;
volatile uint32_t sysDel1Ms;

/* Init SysTick timer */
uint8_t sysInitSystickTimer(void)
{
	if (SysTick_Config((SystemCoreClock / 1000) -1 ))
	{
		return 0;
	}
	return 1;
}

/* Delays 1ms */
void sysDelay1Ms(void)
{
	for (sysDel1Ms = 0; sysDel1Ms < 1000; sysDel1Ms++) { };
}

/* Delays N ms */
void sysDelayMs(uint32_t dly)
{
	sysTiming = dly;

	while (sysTiming > 0) { };

}

/* Counts ticks and decrement delay if needed */
void sysTickCount(void)
{
	sysTicks++;
	if (sysTiming > 0) --sysTiming;	
}

/* Returns the tick count */
uint32_t sysGetTickCount(void)
{
	return sysTicks;
}

/* Init TIM1 and TIM2 */
void sysInitTimer(void)
{
	TIM_TimeBaseInitTypeDef TIM_TimeBaseStructure;
	
	TIM_TimeBaseStructure.TIM_Prescaler = 0;
	TIM_TimeBaseStructure.TIM_CounterMode = TIM_CounterMode_Up;
	TIM_TimeBaseStructure.TIM_Period = 36;
	TIM_TimeBaseStructure.TIM_ClockDivision = 0;
	TIM_TimeBaseStructure.TIM_RepetitionCounter = 0;
	TIM_TimeBaseInit(TIM2, &TIM_TimeBaseStructure);
	
	TIM_ITConfig(TIM2, TIM_IT_Update, ENABLE);
	NVIC_EnableIRQ(TIM2_IRQn);
	NVIC_SetPriority(TIM2_IRQn, 1);
	
	TIM_TimeBaseStructure.TIM_CounterMode = TIM_CounterMode_Up;
	TIM_TimeBaseStructure.TIM_Period = 3265;
	TIM_TimeBaseStructure.TIM_ClockDivision = 0;
	TIM_TimeBaseStructure.TIM_RepetitionCounter = 0;
	TIM_TimeBaseStructure.TIM_Prescaler = 0;
	TIM_TimeBaseInit(TIM1, &TIM_TimeBaseStructure);

	NVIC_EnableIRQ(TIM1_UP_IRQn);
	NVIC_SetPriority(TIM1_UP_IRQn, 1);
	
	TIM_ITConfig(TIM1, TIM_IT_Update, ENABLE);	
	TIM_Cmd(TIM1, ENABLE);
}

/* Stops all the audio timers */
void sysStopTimer(void)
{
	TIM_Cmd(TIM1, DISABLE);
	TIM_Cmd(TIM2, DISABLE);
}

/* Configures clocks */
void sysRCCConfiguration(void)
{
	RCC_APB2PeriphClockCmd(	RCC_APB2Periph_GPIOA | RCC_APB2Periph_GPIOB | RCC_APB2Periph_SPI1 |
							RCC_APB2Periph_AFIO | RCC_APB2Periph_USART1 | RCC_APB2Periph_TIM1, ENABLE);
	
	RCC_APB1PeriphClockCmd( RCC_APB1Periph_TIM2, ENABLE);
	
	RCC_AHBPeriphClockCmd(RCC_AHBPeriph_DMA1, ENABLE);
	
}

/* Basic GPIO initialization */
void sysConfigGPIO(void)
{
	GPIO_InitTypeDef GPIO_InitStructure;

	/* DAC DATA (PA1), CS (PA2) and CLK (PA3) */
	GPIO_InitStructure.GPIO_Pin = GPIO_Pin_1 | GPIO_Pin_2 | GPIO_Pin_3;
	GPIO_InitStructure.GPIO_Mode = GPIO_Mode_Out_PP;
	GPIO_InitStructure.GPIO_Speed = GPIO_Speed_50MHz;
	GPIO_Init(GPIOA, &GPIO_InitStructure);
	
	/* DAC LDAC (PB0) */
	GPIO_InitStructure.GPIO_Pin = GPIO_Pin_0;
	GPIO_Init(GPIOB, &GPIO_InitStructure);
	
	/* ON/OFF button (PA12) */
	GPIO_InitStructure.GPIO_Pin = GPIO_Pin_12;
	GPIO_InitStructure.GPIO_Mode = GPIO_Mode_IPU;
	GPIO_Init(GPIOA, &GPIO_InitStructure);
	
	/* EXTRA button (PB5) */
	GPIO_InitStructure.GPIO_Pin = GPIO_Pin_5;
	GPIO_InitStructure.GPIO_Mode = GPIO_Mode_IPU;
	GPIO_Init(GPIOB, &GPIO_InitStructure);
	
	/* Put DAC CS high */
	GPIOA->BSRR = GPIO_Pin_2;
	
	/* Put LDAC low. We will use the rising edge CS option as LDAC. */
	GPIOB->BRR = GPIO_Pin_0;
}

